# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_LABEL_EXPORT_TABLE_METADATA = Metadata(
    id="6da54211fa44d381c5a103ea7e5e59ecf82e5e80.workbench",
    name="cifti-label-export-table",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiLabelExportTableParamsDictNoTag = typing.TypedDict('_CiftiLabelExportTableParamsDictNoTag', {
    "label-in": InputPathType,
    "map": str,
    "table-out": str,
    "json-out": typing.NotRequired[str | None],
})
CiftiLabelExportTableParamsDictTagged = typing.TypedDict('CiftiLabelExportTableParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-label-export-table"],
    "label-in": InputPathType,
    "map": str,
    "table-out": str,
    "json-out": typing.NotRequired[str | None],
})
CiftiLabelExportTableParamsDict = _CiftiLabelExportTableParamsDictNoTag | CiftiLabelExportTableParamsDictTagged


class CiftiLabelExportTableOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiLabelExportTableParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_label_export_table_params(
    label_in: InputPathType,
    map_: str,
    table_out: str,
    json_out: str | None = None,
) -> CiftiLabelExportTableParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input cifti label file.
        map_: the number or name of the label map to use.
        table_out: output - the output text file.
        json_out: export the hierarchy as json\
            \
            output - filename to write hierarchy to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-label-export-table",
        "label-in": label_in,
        "map": map_,
        "table-out": table_out,
    }
    if json_out is not None:
        params["json-out"] = json_out
    return params


def cifti_label_export_table_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelExportTableParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("map", None) is None:
        raise StyxValidationError("`map` must not be None")
    if not isinstance(params["map"], str):
        raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `str`')
    if params.get("table-out", None) is None:
        raise StyxValidationError("`table-out` must not be None")
    if not isinstance(params["table-out"], str):
        raise StyxValidationError(f'`table-out` has the wrong type: Received `{type(params.get("table-out", None))}` expected `str`')
    if params.get("json-out", None) is not None:
        if not isinstance(params["json-out"], str):
            raise StyxValidationError(f'`json-out` has the wrong type: Received `{type(params.get("json-out", None))}` expected `str | None`')


def cifti_label_export_table_cargs(
    params: CiftiLabelExportTableParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-label-export-table"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("map", None))
    cargs.append(params.get("table-out", None))
    if params.get("json-out", None) is not None:
        cargs.extend([
            "-hierarchy",
            params.get("json-out", None)
        ])
    return cargs


def cifti_label_export_table_outputs(
    params: CiftiLabelExportTableParamsDict,
    execution: Execution,
) -> CiftiLabelExportTableOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelExportTableOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_label_export_table_execute(
    params: CiftiLabelExportTableParamsDict,
    runner: Runner | None = None,
) -> CiftiLabelExportTableOutputs:
    """
    EXPORT LABEL TABLE FROM CIFTI AS TEXT.
    
    Takes the label table from the cifti label map, and writes it to a text
    format matching what is expected by -cifti-label-import.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelExportTableOutputs`).
    """
    cifti_label_export_table_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_LABEL_EXPORT_TABLE_METADATA)
    params = execution.params(params)
    cargs = cifti_label_export_table_cargs(params, execution)
    ret = cifti_label_export_table_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_label_export_table(
    label_in: InputPathType,
    map_: str,
    table_out: str,
    json_out: str | None = None,
    runner: Runner | None = None,
) -> CiftiLabelExportTableOutputs:
    """
    EXPORT LABEL TABLE FROM CIFTI AS TEXT.
    
    Takes the label table from the cifti label map, and writes it to a text
    format matching what is expected by -cifti-label-import.
    
    Args:
        label_in: the input cifti label file.
        map_: the number or name of the label map to use.
        table_out: output - the output text file.
        json_out: export the hierarchy as json\
            \
            output - filename to write hierarchy to.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelExportTableOutputs`).
    """
    params = cifti_label_export_table_params(
        label_in=label_in,
        map_=map_,
        table_out=table_out,
        json_out=json_out,
    )
    return cifti_label_export_table_execute(params, runner)


__all__ = [
    "CIFTI_LABEL_EXPORT_TABLE_METADATA",
    "CiftiLabelExportTableOutputs",
    "CiftiLabelExportTableParamsDict",
    "CiftiLabelExportTableParamsDictTagged",
    "cifti_label_export_table",
    "cifti_label_export_table_execute",
    "cifti_label_export_table_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BORDER_TO_VERTICES_METADATA = Metadata(
    id="3844ea00db2d10878f48538c7a9b40fecdd4a9e0.workbench",
    name="border-to-vertices",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BorderToVerticesParamsDictNoTag = typing.TypedDict('_BorderToVerticesParamsDictNoTag', {
    "surface": InputPathType,
    "border-file": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
})
BorderToVerticesParamsDictTagged = typing.TypedDict('BorderToVerticesParamsDictTagged', {
    "@type": typing.Literal["workbench/border-to-vertices"],
    "surface": InputPathType,
    "border-file": InputPathType,
    "metric-out": str,
    "name": typing.NotRequired[str | None],
})
BorderToVerticesParamsDict = _BorderToVerticesParamsDictNoTag | BorderToVerticesParamsDictTagged


class BorderToVerticesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BorderToVerticesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric file"""


def border_to_vertices_params(
    surface: InputPathType,
    border_file: InputPathType,
    metric_out: str,
    name: str | None = None,
) -> BorderToVerticesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface the borders are drawn on.
        border_file: the border file.
        metric_out: the output metric file.
        name: create ROI for only one border\
            \
            the name of the border.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/border-to-vertices",
        "surface": surface,
        "border-file": border_file,
        "metric-out": metric_out,
    }
    if name is not None:
        params["name"] = name
    return params


def border_to_vertices_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderToVerticesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("border-file", None) is None:
        raise StyxValidationError("`border-file` must not be None")
    if not isinstance(params["border-file"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border-file` has the wrong type: Received `{type(params.get("border-file", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("name", None) is not None:
        if not isinstance(params["name"], str):
            raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", None))}` expected `str | None`')


def border_to_vertices_cargs(
    params: BorderToVerticesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-border-to-vertices"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(execution.input_file(params.get("border-file", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("name", None) is not None:
        cargs.extend([
            "-border",
            params.get("name", None)
        ])
    return cargs


def border_to_vertices_outputs(
    params: BorderToVerticesParamsDict,
    execution: Execution,
) -> BorderToVerticesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BorderToVerticesOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def border_to_vertices_execute(
    params: BorderToVerticesParamsDict,
    runner: Runner | None = None,
) -> BorderToVerticesOutputs:
    """
    DRAW BORDERS AS VERTICES IN A METRIC FILE.
    
    Outputs a metric with 1s on vertices that follow a border, and 0s elsewhere.
    By default, a separate metric column is created for each border.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderToVerticesOutputs`).
    """
    border_to_vertices_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BORDER_TO_VERTICES_METADATA)
    params = execution.params(params)
    cargs = border_to_vertices_cargs(params, execution)
    ret = border_to_vertices_outputs(params, execution)
    execution.run(cargs)
    return ret


def border_to_vertices(
    surface: InputPathType,
    border_file: InputPathType,
    metric_out: str,
    name: str | None = None,
    runner: Runner | None = None,
) -> BorderToVerticesOutputs:
    """
    DRAW BORDERS AS VERTICES IN A METRIC FILE.
    
    Outputs a metric with 1s on vertices that follow a border, and 0s elsewhere.
    By default, a separate metric column is created for each border.
    
    Args:
        surface: the surface the borders are drawn on.
        border_file: the border file.
        metric_out: the output metric file.
        name: create ROI for only one border\
            \
            the name of the border.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderToVerticesOutputs`).
    """
    params = border_to_vertices_params(
        surface=surface,
        border_file=border_file,
        metric_out=metric_out,
        name=name,
    )
    return border_to_vertices_execute(params, runner)


__all__ = [
    "BORDER_TO_VERTICES_METADATA",
    "BorderToVerticesOutputs",
    "BorderToVerticesParamsDict",
    "BorderToVerticesParamsDictTagged",
    "border_to_vertices",
    "border_to_vertices_execute",
    "border_to_vertices_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_MODIFY_KEYS_METADATA = Metadata(
    id="e84f8183032da13712cc4d51e831efb848fef7f4.workbench",
    name="label-modify-keys",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelModifyKeysParamsDictNoTag = typing.TypedDict('_LabelModifyKeysParamsDictNoTag', {
    "label-in": InputPathType,
    "remap-file": str,
    "label-out": str,
    "column": typing.NotRequired[str | None],
})
LabelModifyKeysParamsDictTagged = typing.TypedDict('LabelModifyKeysParamsDictTagged', {
    "@type": typing.Literal["workbench/label-modify-keys"],
    "label-in": InputPathType,
    "remap-file": str,
    "label-out": str,
    "column": typing.NotRequired[str | None],
})
LabelModifyKeysParamsDict = _LabelModifyKeysParamsDictNoTag | LabelModifyKeysParamsDictTagged


class LabelModifyKeysOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelModifyKeysParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """output label file"""


def label_modify_keys_params(
    label_in: InputPathType,
    remap_file: str,
    label_out: str,
    column: str | None = None,
) -> LabelModifyKeysParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input label file.
        remap_file: text file with old and new key values.
        label_out: output label file.
        column: select a single column to use\
            \
            the column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-modify-keys",
        "label-in": label_in,
        "remap-file": remap_file,
        "label-out": label_out,
    }
    if column is not None:
        params["column"] = column
    return params


def label_modify_keys_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelModifyKeysParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("remap-file", None) is None:
        raise StyxValidationError("`remap-file` must not be None")
    if not isinstance(params["remap-file"], str):
        raise StyxValidationError(f'`remap-file` has the wrong type: Received `{type(params.get("remap-file", None))}` expected `str`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')


def label_modify_keys_cargs(
    params: LabelModifyKeysParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-modify-keys"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("remap-file", None))
    cargs.append(params.get("label-out", None))
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    return cargs


def label_modify_keys_outputs(
    params: LabelModifyKeysParamsDict,
    execution: Execution,
) -> LabelModifyKeysOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelModifyKeysOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


def label_modify_keys_execute(
    params: LabelModifyKeysParamsDict,
    runner: Runner | None = None,
) -> LabelModifyKeysOutputs:
    """
    CHANGE KEY VALUES IN A LABEL FILE.
    
    <remap-file> should have lines of the form 'oldkey newkey', like so:
    
    3 5
    5 8
    8 2
    
    This would change the current label with key '3' to use the key '5' instead,
    5 would use 8, and 8 would use 2. Any collision in key values results in the
    label that was not specified in the remap file getting remapped to an
    otherwise unused key. Remapping more than one key to the same new key, or
    the same key to more than one new key, results in an error. This will not
    change the appearance of the file when displayed, as it will change the key
    values in the data at the same time.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelModifyKeysOutputs`).
    """
    label_modify_keys_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_MODIFY_KEYS_METADATA)
    params = execution.params(params)
    cargs = label_modify_keys_cargs(params, execution)
    ret = label_modify_keys_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_modify_keys(
    label_in: InputPathType,
    remap_file: str,
    label_out: str,
    column: str | None = None,
    runner: Runner | None = None,
) -> LabelModifyKeysOutputs:
    """
    CHANGE KEY VALUES IN A LABEL FILE.
    
    <remap-file> should have lines of the form 'oldkey newkey', like so:
    
    3 5
    5 8
    8 2
    
    This would change the current label with key '3' to use the key '5' instead,
    5 would use 8, and 8 would use 2. Any collision in key values results in the
    label that was not specified in the remap file getting remapped to an
    otherwise unused key. Remapping more than one key to the same new key, or
    the same key to more than one new key, results in an error. This will not
    change the appearance of the file when displayed, as it will change the key
    values in the data at the same time.
    
    Args:
        label_in: the input label file.
        remap_file: text file with old and new key values.
        label_out: output label file.
        column: select a single column to use\
            \
            the column number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelModifyKeysOutputs`).
    """
    params = label_modify_keys_params(
        label_in=label_in,
        remap_file=remap_file,
        label_out=label_out,
        column=column,
    )
    return label_modify_keys_execute(params, runner)


__all__ = [
    "LABEL_MODIFY_KEYS_METADATA",
    "LabelModifyKeysOutputs",
    "LabelModifyKeysParamsDict",
    "LabelModifyKeysParamsDictTagged",
    "label_modify_keys",
    "label_modify_keys_execute",
    "label_modify_keys_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

ANNOTATION_RESAMPLE_METADATA = Metadata(
    id="a7b40c2eabbd684e2841e97eb61c046f68f16b83.workbench",
    name="annotation-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_AnnotationResampleSurfacePairParamsDictNoTag = typing.TypedDict('_AnnotationResampleSurfacePairParamsDictNoTag', {
    "source-surface": InputPathType,
    "target-surface": InputPathType,
})
AnnotationResampleSurfacePairParamsDictTagged = typing.TypedDict('AnnotationResampleSurfacePairParamsDictTagged', {
    "@type": typing.Literal["surface-pair"],
    "source-surface": InputPathType,
    "target-surface": InputPathType,
})
AnnotationResampleSurfacePairParamsDict = _AnnotationResampleSurfacePairParamsDictNoTag | AnnotationResampleSurfacePairParamsDictTagged


_AnnotationResampleParamsDictNoTag = typing.TypedDict('_AnnotationResampleParamsDictNoTag', {
    "annotation-in": InputPathType,
    "annotation-out": str,
    "surface-pair": typing.NotRequired[list[AnnotationResampleSurfacePairParamsDict] | None],
})
AnnotationResampleParamsDictTagged = typing.TypedDict('AnnotationResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/annotation-resample"],
    "annotation-in": InputPathType,
    "annotation-out": str,
    "surface-pair": typing.NotRequired[list[AnnotationResampleSurfacePairParamsDict] | None],
})
AnnotationResampleParamsDict = _AnnotationResampleParamsDictNoTag | AnnotationResampleParamsDictTagged


def annotation_resample_surface_pair(
    source_surface: InputPathType,
    target_surface: InputPathType,
) -> AnnotationResampleSurfacePairParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source_surface: the midthickness surface of the current mesh the\
            annotations use.
        target_surface: the midthickness surface of the mesh the annotations\
            should be transferred to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "surface-pair",
        "source-surface": source_surface,
        "target-surface": target_surface,
    }
    return params


def annotation_resample_surface_pair_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AnnotationResampleSurfacePairParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-surface", None) is None:
        raise StyxValidationError("`source-surface` must not be None")
    if not isinstance(params["source-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`source-surface` has the wrong type: Received `{type(params.get("source-surface", None))}` expected `InputPathType`')
    if params.get("target-surface", None) is None:
        raise StyxValidationError("`target-surface` must not be None")
    if not isinstance(params["target-surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`target-surface` has the wrong type: Received `{type(params.get("target-surface", None))}` expected `InputPathType`')


def annotation_resample_surface_pair_cargs(
    params: AnnotationResampleSurfacePairParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-surface-pair",
        execution.input_file(params.get("source-surface", None)),
        execution.input_file(params.get("target-surface", None))
    ])
    return cargs


class AnnotationResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `AnnotationResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def annotation_resample_params(
    annotation_in: InputPathType,
    annotation_out: str,
    surface_pair: list[AnnotationResampleSurfacePairParamsDict] | None = None,
) -> AnnotationResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        annotation_in: the annotation file to resample.
        annotation_out: name of resampled annotation file.
        surface_pair: pair of surfaces for resampling surface annotations for\
            one structure.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/annotation-resample",
        "annotation-in": annotation_in,
        "annotation-out": annotation_out,
    }
    if surface_pair is not None:
        params["surface-pair"] = surface_pair
    return params


def annotation_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `AnnotationResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("annotation-in", None) is None:
        raise StyxValidationError("`annotation-in` must not be None")
    if not isinstance(params["annotation-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`annotation-in` has the wrong type: Received `{type(params.get("annotation-in", None))}` expected `InputPathType`')
    if params.get("annotation-out", None) is None:
        raise StyxValidationError("`annotation-out` must not be None")
    if not isinstance(params["annotation-out"], str):
        raise StyxValidationError(f'`annotation-out` has the wrong type: Received `{type(params.get("annotation-out", None))}` expected `str`')
    if params.get("surface-pair", None) is not None:
        if not isinstance(params["surface-pair"], list):
            raise StyxValidationError(f'`surface-pair` has the wrong type: Received `{type(params.get("surface-pair", None))}` expected `list[AnnotationResampleSurfacePairParamsDict] | None`')
        for e in params["surface-pair"]:
            annotation_resample_surface_pair_validate(e)


def annotation_resample_cargs(
    params: AnnotationResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-annotation-resample"
    ])
    cargs.append(execution.input_file(params.get("annotation-in", None)))
    cargs.append(params.get("annotation-out", None))
    if params.get("surface-pair", None) is not None:
        cargs.extend([a for c in [annotation_resample_surface_pair_cargs(s, execution) for s in params.get("surface-pair", None)] for a in c])
    return cargs


def annotation_resample_outputs(
    params: AnnotationResampleParamsDict,
    execution: Execution,
) -> AnnotationResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = AnnotationResampleOutputs(
        root=execution.output_file("."),
    )
    return ret


def annotation_resample_execute(
    params: AnnotationResampleParamsDict,
    runner: Runner | None = None,
) -> AnnotationResampleOutputs:
    """
    RESAMPLE AN ANNOTATION FILE TO DIFFERENT MESHES.
    
    Resample an annotation file from the source mesh to the target mesh.
    
    Only annotations in surface space are modified, no changes are made to
    annotations in other spaces. The -surface-pair option may be repeated for
    additional structures used by surface space annotations.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AnnotationResampleOutputs`).
    """
    annotation_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(ANNOTATION_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = annotation_resample_cargs(params, execution)
    ret = annotation_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def annotation_resample(
    annotation_in: InputPathType,
    annotation_out: str,
    surface_pair: list[AnnotationResampleSurfacePairParamsDict] | None = None,
    runner: Runner | None = None,
) -> AnnotationResampleOutputs:
    """
    RESAMPLE AN ANNOTATION FILE TO DIFFERENT MESHES.
    
    Resample an annotation file from the source mesh to the target mesh.
    
    Only annotations in surface space are modified, no changes are made to
    annotations in other spaces. The -surface-pair option may be repeated for
    additional structures used by surface space annotations.
    
    Args:
        annotation_in: the annotation file to resample.
        annotation_out: name of resampled annotation file.
        surface_pair: pair of surfaces for resampling surface annotations for\
            one structure.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `AnnotationResampleOutputs`).
    """
    params = annotation_resample_params(
        annotation_in=annotation_in,
        annotation_out=annotation_out,
        surface_pair=surface_pair,
    )
    return annotation_resample_execute(params, runner)


__all__ = [
    "ANNOTATION_RESAMPLE_METADATA",
    "AnnotationResampleOutputs",
    "AnnotationResampleParamsDict",
    "AnnotationResampleParamsDictTagged",
    "AnnotationResampleSurfacePairParamsDict",
    "AnnotationResampleSurfacePairParamsDictTagged",
    "annotation_resample",
    "annotation_resample_execute",
    "annotation_resample_params",
    "annotation_resample_surface_pair",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_DISTORTION_METADATA = Metadata(
    id="71b5b8e30a3f86c052e4af8965fbf9041a247504.workbench",
    name="volume-distortion",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeDistortionParamsDictNoTag = typing.TypedDict('_VolumeDistortionParamsDictNoTag', {
    "warpfield": str,
    "volume-out": str,
    "source-volume": typing.NotRequired[str | None],
    "log2": bool,
    "circular": bool,
})
VolumeDistortionParamsDictTagged = typing.TypedDict('VolumeDistortionParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-distortion"],
    "warpfield": str,
    "volume-out": str,
    "source-volume": typing.NotRequired[str | None],
    "log2": bool,
    "circular": bool,
})
VolumeDistortionParamsDict = _VolumeDistortionParamsDictNoTag | VolumeDistortionParamsDictTagged


class VolumeDistortionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeDistortionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output distortion measures"""


def volume_distortion_params(
    warpfield: str,
    volume_out: str,
    source_volume: str | None = None,
    log2: bool = False,
    circular: bool = False,
) -> VolumeDistortionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        warpfield: the warpfield to compute the distortion of.
        volume_out: the output distortion measures.
        source_volume: MUST be used if using a fnirt warpfield\
            \
            the source volume used when generating the warpfield.
        log2: apply base-2 log transform.
        circular: use the circle-based formula for the anisotropic measure.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-distortion",
        "warpfield": warpfield,
        "volume-out": volume_out,
        "log2": log2,
        "circular": circular,
    }
    if source_volume is not None:
        params["source-volume"] = source_volume
    return params


def volume_distortion_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeDistortionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("warpfield", None) is None:
        raise StyxValidationError("`warpfield` must not be None")
    if not isinstance(params["warpfield"], str):
        raise StyxValidationError(f'`warpfield` has the wrong type: Received `{type(params.get("warpfield", None))}` expected `str`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("source-volume", None) is not None:
        if not isinstance(params["source-volume"], str):
            raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str | None`')
    if params.get("log2", False) is None:
        raise StyxValidationError("`log2` must not be None")
    if not isinstance(params["log2"], bool):
        raise StyxValidationError(f'`log2` has the wrong type: Received `{type(params.get("log2", False))}` expected `bool`')
    if params.get("circular", False) is None:
        raise StyxValidationError("`circular` must not be None")
    if not isinstance(params["circular"], bool):
        raise StyxValidationError(f'`circular` has the wrong type: Received `{type(params.get("circular", False))}` expected `bool`')


def volume_distortion_cargs(
    params: VolumeDistortionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-distortion"
    ])
    cargs.append(params.get("warpfield", None))
    cargs.append(params.get("volume-out", None))
    if params.get("source-volume", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("source-volume", None)
        ])
    if params.get("log2", False):
        cargs.append("-log2")
    if params.get("circular", False):
        cargs.append("-circular")
    return cargs


def volume_distortion_outputs(
    params: VolumeDistortionParamsDict,
    execution: Execution,
) -> VolumeDistortionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeDistortionOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_distortion_execute(
    params: VolumeDistortionParamsDict,
    runner: Runner | None = None,
) -> VolumeDistortionOutputs:
    """
    CALCULATE VOLUME WARPFIELD DISTORTION.
    
    Calculates isotropic and anisotropic distortions in the volume warpfield. At
    each voxel, the gradient of the absolute warpfield is computed to obtain the
    local affine transforms for each voxel (jacobian matrices), and strain
    tensors are derived from them. The isotropic component (volumetric expansion
    ratio) is the product of the three principal strains. The default measure
    ('elongation') for the anisotropic component is the largest principal strain
    divided by the smallest.
    
    The -circular option instead calculates the anisotropic component by
    transforming the principal strains into log space, considering them as
    x-values of points on a circle 120 degrees apart, finds the circle's
    diameter, and transforms that back to a ratio.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeDistortionOutputs`).
    """
    volume_distortion_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_DISTORTION_METADATA)
    params = execution.params(params)
    cargs = volume_distortion_cargs(params, execution)
    ret = volume_distortion_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_distortion(
    warpfield: str,
    volume_out: str,
    source_volume: str | None = None,
    log2: bool = False,
    circular: bool = False,
    runner: Runner | None = None,
) -> VolumeDistortionOutputs:
    """
    CALCULATE VOLUME WARPFIELD DISTORTION.
    
    Calculates isotropic and anisotropic distortions in the volume warpfield. At
    each voxel, the gradient of the absolute warpfield is computed to obtain the
    local affine transforms for each voxel (jacobian matrices), and strain
    tensors are derived from them. The isotropic component (volumetric expansion
    ratio) is the product of the three principal strains. The default measure
    ('elongation') for the anisotropic component is the largest principal strain
    divided by the smallest.
    
    The -circular option instead calculates the anisotropic component by
    transforming the principal strains into log space, considering them as
    x-values of points on a circle 120 degrees apart, finds the circle's
    diameter, and transforms that back to a ratio.
    
    Args:
        warpfield: the warpfield to compute the distortion of.
        volume_out: the output distortion measures.
        source_volume: MUST be used if using a fnirt warpfield\
            \
            the source volume used when generating the warpfield.
        log2: apply base-2 log transform.
        circular: use the circle-based formula for the anisotropic measure.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeDistortionOutputs`).
    """
    params = volume_distortion_params(
        warpfield=warpfield,
        volume_out=volume_out,
        source_volume=source_volume,
        log2=log2,
        circular=circular,
    )
    return volume_distortion_execute(params, runner)


__all__ = [
    "VOLUME_DISTORTION_METADATA",
    "VolumeDistortionOutputs",
    "VolumeDistortionParamsDict",
    "VolumeDistortionParamsDictTagged",
    "volume_distortion",
    "volume_distortion_execute",
    "volume_distortion_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_FLIP_LR_METADATA = Metadata(
    id="bdd0276d22f7add2db023bdd235def55dde8330f.workbench",
    name="surface-flip-lr",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceFlipLrParamsDictNoTag = typing.TypedDict('_SurfaceFlipLrParamsDictNoTag', {
    "surface": InputPathType,
    "surface-out": str,
})
SurfaceFlipLrParamsDictTagged = typing.TypedDict('SurfaceFlipLrParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-flip-lr"],
    "surface": InputPathType,
    "surface-out": str,
})
SurfaceFlipLrParamsDict = _SurfaceFlipLrParamsDictNoTag | SurfaceFlipLrParamsDictTagged


class SurfaceFlipLrOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceFlipLrParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the output flipped surface"""


def surface_flip_lr_params(
    surface: InputPathType,
    surface_out: str,
) -> SurfaceFlipLrParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to flip.
        surface_out: the output flipped surface.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-flip-lr",
        "surface": surface,
        "surface-out": surface_out,
    }
    return params


def surface_flip_lr_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceFlipLrParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_flip_lr_cargs(
    params: SurfaceFlipLrParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-flip-lr"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_flip_lr_outputs(
    params: SurfaceFlipLrParamsDict,
    execution: Execution,
) -> SurfaceFlipLrOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceFlipLrOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_flip_lr_execute(
    params: SurfaceFlipLrParamsDict,
    runner: Runner | None = None,
) -> SurfaceFlipLrOutputs:
    """
    MIRROR A SURFACE THROUGH THE YZ PLANE.
    
    This command negates the x coordinate of each vertex, and flips the surface
    normals, so that you have a surface of opposite handedness with the same
    features and vertex correspondence, with normals consistent with the
    original surface. That is, if the input surface has normals facing outward,
    the output surface will also have normals facing outward.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceFlipLrOutputs`).
    """
    surface_flip_lr_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_FLIP_LR_METADATA)
    params = execution.params(params)
    cargs = surface_flip_lr_cargs(params, execution)
    ret = surface_flip_lr_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_flip_lr(
    surface: InputPathType,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceFlipLrOutputs:
    """
    MIRROR A SURFACE THROUGH THE YZ PLANE.
    
    This command negates the x coordinate of each vertex, and flips the surface
    normals, so that you have a surface of opposite handedness with the same
    features and vertex correspondence, with normals consistent with the
    original surface. That is, if the input surface has normals facing outward,
    the output surface will also have normals facing outward.
    
    Args:
        surface: the surface to flip.
        surface_out: the output flipped surface.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceFlipLrOutputs`).
    """
    params = surface_flip_lr_params(
        surface=surface,
        surface_out=surface_out,
    )
    return surface_flip_lr_execute(params, runner)


__all__ = [
    "SURFACE_FLIP_LR_METADATA",
    "SurfaceFlipLrOutputs",
    "SurfaceFlipLrParamsDict",
    "SurfaceFlipLrParamsDictTagged",
    "surface_flip_lr",
    "surface_flip_lr_execute",
    "surface_flip_lr_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_ESTIMATE_FWHM_METADATA = Metadata(
    id="25607ae5dff68c55973abc5a13016a50bb089e8e.workbench",
    name="volume-estimate-fwhm",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeEstimateFwhmParamsDictNoTag = typing.TypedDict('_VolumeEstimateFwhmParamsDictNoTag', {
    "volume": InputPathType,
    "demean": typing.NotRequired[bool | None],
    "subvol": typing.NotRequired[str | None],
    "roivol": typing.NotRequired[InputPathType | None],
})
VolumeEstimateFwhmParamsDictTagged = typing.TypedDict('VolumeEstimateFwhmParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-estimate-fwhm"],
    "volume": InputPathType,
    "demean": typing.NotRequired[bool | None],
    "subvol": typing.NotRequired[str | None],
    "roivol": typing.NotRequired[InputPathType | None],
})
VolumeEstimateFwhmParamsDict = _VolumeEstimateFwhmParamsDictNoTag | VolumeEstimateFwhmParamsDictTagged


class VolumeEstimateFwhmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeEstimateFwhmParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_estimate_fwhm_params(
    volume: InputPathType,
    demean: bool | None = None,
    subvol: str | None = None,
    roivol: InputPathType | None = None,
) -> VolumeEstimateFwhmParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the input volume.
        demean: estimate for the whole file at once, not each subvolume\
            separately\
            \
            subtract the mean image before estimating smoothness.
        subvol: select a single subvolume to estimate smoothness of\
            \
            the subvolume number or name.
        roivol: use only data within an ROI\
            \
            the volume to use as an ROI.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-estimate-fwhm",
        "volume": volume,
    }
    if demean is not None:
        params["demean"] = demean
    if subvol is not None:
        params["subvol"] = subvol
    if roivol is not None:
        params["roivol"] = roivol
    return params


def volume_estimate_fwhm_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeEstimateFwhmParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("demean", None) is not None:
        if not isinstance(params["demean"], bool):
            raise StyxValidationError(f'`demean` has the wrong type: Received `{type(params.get("demean", None))}` expected `bool | None`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roivol", None) is not None:
        if not isinstance(params["roivol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roivol` has the wrong type: Received `{type(params.get("roivol", None))}` expected `InputPathType | None`')


def volume_estimate_fwhm_cargs(
    params: VolumeEstimateFwhmParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-estimate-fwhm"
    ])
    cargs.append(execution.input_file(params.get("volume", None)))
    if params.get("demean", None) is not None:
        cargs.extend([
            "-whole-file",
            "-demean"
        ])
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roivol", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roivol", None))
        ])
    return cargs


def volume_estimate_fwhm_outputs(
    params: VolumeEstimateFwhmParamsDict,
    execution: Execution,
) -> VolumeEstimateFwhmOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeEstimateFwhmOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_estimate_fwhm_execute(
    params: VolumeEstimateFwhmParamsDict,
    runner: Runner | None = None,
) -> VolumeEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A VOLUME.
    
    Estimates the smoothness of the input volume in X, Y, and Z directions
    separately, printing the estimates to standard output, in mm as FWHM. If
    -subvolume or -whole-file are not specified, each subvolume is estimated and
    displayed separately.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeEstimateFwhmOutputs`).
    """
    volume_estimate_fwhm_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_ESTIMATE_FWHM_METADATA)
    params = execution.params(params)
    cargs = volume_estimate_fwhm_cargs(params, execution)
    ret = volume_estimate_fwhm_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_estimate_fwhm(
    volume: InputPathType,
    demean: bool | None = None,
    subvol: str | None = None,
    roivol: InputPathType | None = None,
    runner: Runner | None = None,
) -> VolumeEstimateFwhmOutputs:
    """
    ESTIMATE FWHM SMOOTHNESS OF A VOLUME.
    
    Estimates the smoothness of the input volume in X, Y, and Z directions
    separately, printing the estimates to standard output, in mm as FWHM. If
    -subvolume or -whole-file are not specified, each subvolume is estimated and
    displayed separately.
    
    Args:
        volume: the input volume.
        demean: estimate for the whole file at once, not each subvolume\
            separately\
            \
            subtract the mean image before estimating smoothness.
        subvol: select a single subvolume to estimate smoothness of\
            \
            the subvolume number or name.
        roivol: use only data within an ROI\
            \
            the volume to use as an ROI.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeEstimateFwhmOutputs`).
    """
    params = volume_estimate_fwhm_params(
        volume=volume,
        demean=demean,
        subvol=subvol,
        roivol=roivol,
    )
    return volume_estimate_fwhm_execute(params, runner)


__all__ = [
    "VOLUME_ESTIMATE_FWHM_METADATA",
    "VolumeEstimateFwhmOutputs",
    "VolumeEstimateFwhmParamsDict",
    "VolumeEstimateFwhmParamsDictTagged",
    "volume_estimate_fwhm",
    "volume_estimate_fwhm_execute",
    "volume_estimate_fwhm_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_CUT_RESAMPLE_METADATA = Metadata(
    id="4fdc0efb8ff92dfb91c639b6b9fe6d92fc4d0e35.workbench",
    name="surface-cut-resample",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceCutResampleParamsDictNoTag = typing.TypedDict('_SurfaceCutResampleParamsDictNoTag', {
    "surface-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "surface-out": str,
})
SurfaceCutResampleParamsDictTagged = typing.TypedDict('SurfaceCutResampleParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-cut-resample"],
    "surface-in": InputPathType,
    "current-sphere": InputPathType,
    "new-sphere": InputPathType,
    "surface-out": str,
})
SurfaceCutResampleParamsDict = _SurfaceCutResampleParamsDictNoTag | SurfaceCutResampleParamsDictTagged


class SurfaceCutResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceCutResampleParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the output surface file"""


def surface_cut_resample_params(
    surface_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    surface_out: str,
) -> SurfaceCutResampleParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_in: the surface file to resample.
        current_sphere: a sphere surface with the mesh that the input surface\
            is currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        surface_out: the output surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-cut-resample",
        "surface-in": surface_in,
        "current-sphere": current_sphere,
        "new-sphere": new_sphere,
        "surface-out": surface_out,
    }
    return params


def surface_cut_resample_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceCutResampleParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-in", None) is None:
        raise StyxValidationError("`surface-in` must not be None")
    if not isinstance(params["surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-in` has the wrong type: Received `{type(params.get("surface-in", None))}` expected `InputPathType`')
    if params.get("current-sphere", None) is None:
        raise StyxValidationError("`current-sphere` must not be None")
    if not isinstance(params["current-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`current-sphere` has the wrong type: Received `{type(params.get("current-sphere", None))}` expected `InputPathType`')
    if params.get("new-sphere", None) is None:
        raise StyxValidationError("`new-sphere` must not be None")
    if not isinstance(params["new-sphere"], (pathlib.Path, str)):
        raise StyxValidationError(f'`new-sphere` has the wrong type: Received `{type(params.get("new-sphere", None))}` expected `InputPathType`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_cut_resample_cargs(
    params: SurfaceCutResampleParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-cut-resample"
    ])
    cargs.append(execution.input_file(params.get("surface-in", None)))
    cargs.append(execution.input_file(params.get("current-sphere", None)))
    cargs.append(execution.input_file(params.get("new-sphere", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_cut_resample_outputs(
    params: SurfaceCutResampleParamsDict,
    execution: Execution,
) -> SurfaceCutResampleOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceCutResampleOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_cut_resample_execute(
    params: SurfaceCutResampleParamsDict,
    runner: Runner | None = None,
) -> SurfaceCutResampleOutputs:
    """
    RESAMPLE A CUT SURFACE.
    
    Resamples a surface file, given two spherical surfaces that are in register.
    Barycentric resampling is used, because it is usually better for resampling
    surfaces, and because it is needed to figure out the new topology anyway.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCutResampleOutputs`).
    """
    surface_cut_resample_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_CUT_RESAMPLE_METADATA)
    params = execution.params(params)
    cargs = surface_cut_resample_cargs(params, execution)
    ret = surface_cut_resample_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_cut_resample(
    surface_in: InputPathType,
    current_sphere: InputPathType,
    new_sphere: InputPathType,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceCutResampleOutputs:
    """
    RESAMPLE A CUT SURFACE.
    
    Resamples a surface file, given two spherical surfaces that are in register.
    Barycentric resampling is used, because it is usually better for resampling
    surfaces, and because it is needed to figure out the new topology anyway.
    
    Args:
        surface_in: the surface file to resample.
        current_sphere: a sphere surface with the mesh that the input surface\
            is currently on.
        new_sphere: a sphere surface that is in register with <current-sphere>\
            and has the desired output mesh.
        surface_out: the output surface file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCutResampleOutputs`).
    """
    params = surface_cut_resample_params(
        surface_in=surface_in,
        current_sphere=current_sphere,
        new_sphere=new_sphere,
        surface_out=surface_out,
    )
    return surface_cut_resample_execute(params, runner)


__all__ = [
    "SURFACE_CUT_RESAMPLE_METADATA",
    "SurfaceCutResampleOutputs",
    "SurfaceCutResampleParamsDict",
    "SurfaceCutResampleParamsDictTagged",
    "surface_cut_resample",
    "surface_cut_resample_execute",
    "surface_cut_resample_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_SMOOTHING_METADATA = Metadata(
    id="d799671856dffbf8bcd37b5fc4f2635099c3c22c.workbench",
    name="volume-smoothing",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeSmoothingParamsDictNoTag = typing.TypedDict('_VolumeSmoothingParamsDictNoTag', {
    "volume-in": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "roivol": typing.NotRequired[InputPathType | None],
    "fix-zeros": bool,
    "fwhm": bool,
})
VolumeSmoothingParamsDictTagged = typing.TypedDict('VolumeSmoothingParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-smoothing"],
    "volume-in": InputPathType,
    "kernel": float,
    "volume-out": str,
    "subvol": typing.NotRequired[str | None],
    "roivol": typing.NotRequired[InputPathType | None],
    "fix-zeros": bool,
    "fwhm": bool,
})
VolumeSmoothingParamsDict = _VolumeSmoothingParamsDictNoTag | VolumeSmoothingParamsDictTagged


class VolumeSmoothingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeSmoothingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_smoothing_params(
    volume_in: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    roivol: InputPathType | None = None,
    fix_zeros: bool = False,
    fwhm: bool = False,
) -> VolumeSmoothingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the volume to smooth.
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        volume_out: the output volume.
        subvol: select a single subvolume to smooth\
            \
            the subvolume number or name.
        roivol: smooth only from data within an ROI\
            \
            the volume to use as an ROI.
        fix_zeros: treat zero values as not being data.
        fwhm: kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-smoothing",
        "volume-in": volume_in,
        "kernel": kernel,
        "volume-out": volume_out,
        "fix-zeros": fix_zeros,
        "fwhm": fwhm,
    }
    if subvol is not None:
        params["subvol"] = subvol
    if roivol is not None:
        params["roivol"] = roivol
    return params


def volume_smoothing_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeSmoothingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("subvol", None) is not None:
        if not isinstance(params["subvol"], str):
            raise StyxValidationError(f'`subvol` has the wrong type: Received `{type(params.get("subvol", None))}` expected `str | None`')
    if params.get("roivol", None) is not None:
        if not isinstance(params["roivol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roivol` has the wrong type: Received `{type(params.get("roivol", None))}` expected `InputPathType | None`')
    if params.get("fix-zeros", False) is None:
        raise StyxValidationError("`fix-zeros` must not be None")
    if not isinstance(params["fix-zeros"], bool):
        raise StyxValidationError(f'`fix-zeros` has the wrong type: Received `{type(params.get("fix-zeros", False))}` expected `bool`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_smoothing_cargs(
    params: VolumeSmoothingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-smoothing"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(str(params.get("kernel", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("subvol", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvol", None)
        ])
    if params.get("roivol", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roivol", None))
        ])
    if params.get("fix-zeros", False):
        cargs.append("-fix-zeros")
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def volume_smoothing_outputs(
    params: VolumeSmoothingParamsDict,
    execution: Execution,
) -> VolumeSmoothingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeSmoothingOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_smoothing_execute(
    params: VolumeSmoothingParamsDict,
    runner: Runner | None = None,
) -> VolumeSmoothingOutputs:
    """
    SMOOTH A VOLUME FILE.
    
    Gaussian smoothing for volumes. By default, smooths all subvolumes with no
    ROI, if ROI is given, only positive voxels in the ROI volume have their
    values used, and all other voxels are set to zero. Smoothing a
    non-orthogonal volume will be significantly slower, because the operation
    cannot be separated into 1-dimensional smoothings without distorting the
    kernel shape.
    
    The -fix-zeros option causes the smoothing to not use an input value if it
    is zero, but still write a smoothed value to the voxel. This is useful for
    zeros that indicate lack of information, preventing them from pulling down
    the intensity of nearby voxels, while giving the zero an extrapolated value.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSmoothingOutputs`).
    """
    volume_smoothing_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_SMOOTHING_METADATA)
    params = execution.params(params)
    cargs = volume_smoothing_cargs(params, execution)
    ret = volume_smoothing_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_smoothing(
    volume_in: InputPathType,
    kernel: float,
    volume_out: str,
    subvol: str | None = None,
    roivol: InputPathType | None = None,
    fix_zeros: bool = False,
    fwhm: bool = False,
    runner: Runner | None = None,
) -> VolumeSmoothingOutputs:
    """
    SMOOTH A VOLUME FILE.
    
    Gaussian smoothing for volumes. By default, smooths all subvolumes with no
    ROI, if ROI is given, only positive voxels in the ROI volume have their
    values used, and all other voxels are set to zero. Smoothing a
    non-orthogonal volume will be significantly slower, because the operation
    cannot be separated into 1-dimensional smoothings without distorting the
    kernel shape.
    
    The -fix-zeros option causes the smoothing to not use an input value if it
    is zero, but still write a smoothed value to the voxel. This is useful for
    zeros that indicate lack of information, preventing them from pulling down
    the intensity of nearby voxels, while giving the zero an extrapolated value.
    
    Args:
        volume_in: the volume to smooth.
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        volume_out: the output volume.
        subvol: select a single subvolume to smooth\
            \
            the subvolume number or name.
        roivol: smooth only from data within an ROI\
            \
            the volume to use as an ROI.
        fix_zeros: treat zero values as not being data.
        fwhm: kernel size is FWHM, not sigma.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSmoothingOutputs`).
    """
    params = volume_smoothing_params(
        volume_in=volume_in,
        kernel=kernel,
        volume_out=volume_out,
        subvol=subvol,
        roivol=roivol,
        fix_zeros=fix_zeros,
        fwhm=fwhm,
    )
    return volume_smoothing_execute(params, runner)


__all__ = [
    "VOLUME_SMOOTHING_METADATA",
    "VolumeSmoothingOutputs",
    "VolumeSmoothingParamsDict",
    "VolumeSmoothingParamsDictTagged",
    "volume_smoothing",
    "volume_smoothing_execute",
    "volume_smoothing_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_MERGE_METADATA = Metadata(
    id="7fb02481d87d8cd2e3909083db81c14bb985ba00.workbench",
    name="cifti-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiMergeUpToParamsDictNoTag = typing.TypedDict('_CiftiMergeUpToParamsDictNoTag', {
    "last-index": str,
    "reverse": bool,
})
CiftiMergeUpToParamsDictTagged = typing.TypedDict('CiftiMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-index": str,
    "reverse": bool,
})
CiftiMergeUpToParamsDict = _CiftiMergeUpToParamsDictNoTag | CiftiMergeUpToParamsDictTagged


_CiftiMergeIndexParamsDictNoTag = typing.TypedDict('_CiftiMergeIndexParamsDictNoTag', {
    "index": str,
    "up-to": typing.NotRequired[CiftiMergeUpToParamsDict | None],
})
CiftiMergeIndexParamsDictTagged = typing.TypedDict('CiftiMergeIndexParamsDictTagged', {
    "@type": typing.Literal["index"],
    "index": str,
    "up-to": typing.NotRequired[CiftiMergeUpToParamsDict | None],
})
CiftiMergeIndexParamsDict = _CiftiMergeIndexParamsDictNoTag | CiftiMergeIndexParamsDictTagged


_CiftiMergeCiftiParamsDictNoTag = typing.TypedDict('_CiftiMergeCiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
    "index": typing.NotRequired[list[CiftiMergeIndexParamsDict] | None],
})
CiftiMergeCiftiParamsDictTagged = typing.TypedDict('CiftiMergeCiftiParamsDictTagged', {
    "@type": typing.Literal["cifti"],
    "cifti-in": InputPathType,
    "index": typing.NotRequired[list[CiftiMergeIndexParamsDict] | None],
})
CiftiMergeCiftiParamsDict = _CiftiMergeCiftiParamsDictNoTag | CiftiMergeCiftiParamsDictTagged


_CiftiMergeParamsDictNoTag = typing.TypedDict('_CiftiMergeParamsDictNoTag', {
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiMergeCiftiParamsDict] | None],
    "limit-GB": typing.NotRequired[float | None],
    "direction": typing.NotRequired[str | None],
})
CiftiMergeParamsDictTagged = typing.TypedDict('CiftiMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-merge"],
    "cifti-out": str,
    "cifti": typing.NotRequired[list[CiftiMergeCiftiParamsDict] | None],
    "limit-GB": typing.NotRequired[float | None],
    "direction": typing.NotRequired[str | None],
})
CiftiMergeParamsDict = _CiftiMergeParamsDictNoTag | CiftiMergeParamsDictTagged


def cifti_merge_up_to(
    last_index: str,
    reverse: bool = False,
) -> CiftiMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_index: the number or name of the last index to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-index": last_index,
        "reverse": reverse,
    }
    return params


def cifti_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-index", None) is None:
        raise StyxValidationError("`last-index` must not be None")
    if not isinstance(params["last-index"], str):
        raise StyxValidationError(f'`last-index` has the wrong type: Received `{type(params.get("last-index", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def cifti_merge_up_to_cargs(
    params: CiftiMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-index", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def cifti_merge_index(
    index: str,
    up_to: CiftiMergeUpToParamsDict | None = None,
) -> CiftiMergeIndexParamsDictTagged:
    """
    Build parameters.
    
    Args:
        index: the index number (starting from 1), or name.
        up_to: use an inclusive range of indices.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "index",
        "index": index,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def cifti_merge_index_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeIndexParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("index", None) is None:
        raise StyxValidationError("`index` must not be None")
    if not isinstance(params["index"], str):
        raise StyxValidationError(f'`index` has the wrong type: Received `{type(params.get("index", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        cifti_merge_up_to_validate(params["up-to"])


def cifti_merge_index_cargs(
    params: CiftiMergeIndexParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-index",
        params.get("index", None),
        *(cifti_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def cifti_merge_cifti(
    cifti_in: InputPathType,
    index: list[CiftiMergeIndexParamsDict] | None = None,
) -> CiftiMergeCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: a cifti file to use data from.
        index: select a single index to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti-in": cifti_in,
    }
    if index is not None:
        params["index"] = index
    return params


def cifti_merge_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("index", None) is not None:
        if not isinstance(params["index"], list):
            raise StyxValidationError(f'`index` has the wrong type: Received `{type(params.get("index", None))}` expected `list[CiftiMergeIndexParamsDict] | None`')
        for e in params["index"]:
            cifti_merge_index_validate(e)


def cifti_merge_cifti_cargs(
    params: CiftiMergeCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti",
        execution.input_file(params.get("cifti-in", None)),
        *([a for c in [cifti_merge_index_cargs(s, execution) for s in params.get("index", None)] for a in c] if (params.get("index", None) is not None) else [])
    ])
    return cargs


class CiftiMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_merge_params(
    cifti_out: str,
    cifti: list[CiftiMergeCiftiParamsDict] | None = None,
    limit_gb: float | None = None,
    direction: str | None = None,
) -> CiftiMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: output cifti file.
        cifti: specify an input cifti file.
        limit_gb: restrict memory used for file reading efficiency\
            \
            memory limit in gigabytes.
        direction: merge in a direction other than along rows\
            \
            the dimension to split/concatenate along, default ROW.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-merge",
        "cifti-out": cifti_out,
    }
    if cifti is not None:
        params["cifti"] = cifti
    if limit_gb is not None:
        params["limit-GB"] = limit_gb
    if direction is not None:
        params["direction"] = direction
    return params


def cifti_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("cifti", None) is not None:
        if not isinstance(params["cifti"], list):
            raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `list[CiftiMergeCiftiParamsDict] | None`')
        for e in params["cifti"]:
            cifti_merge_cifti_validate(e)
    if params.get("limit-GB", None) is not None:
        if not isinstance(params["limit-GB"], (float, int)):
            raise StyxValidationError(f'`limit-GB` has the wrong type: Received `{type(params.get("limit-GB", None))}` expected `float | None`')
    if params.get("direction", None) is not None:
        if not isinstance(params["direction"], str):
            raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str | None`')


def cifti_merge_cargs(
    params: CiftiMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-merge"
    ])
    cargs.append(params.get("cifti-out", None))
    if params.get("cifti", None) is not None:
        cargs.extend([a for c in [cifti_merge_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c])
    if params.get("limit-GB", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("limit-GB", None))
        ])
    if params.get("direction", None) is not None:
        cargs.extend([
            "-direction",
            params.get("direction", None)
        ])
    return cargs


def cifti_merge_outputs(
    params: CiftiMergeParamsDict,
    execution: Execution,
) -> CiftiMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiMergeOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_merge_execute(
    params: CiftiMergeParamsDict,
    runner: Runner | None = None,
) -> CiftiMergeOutputs:
    """
    MERGE OR SPLIT ON SERIES, SCALAR, OR LABEL DIMENSIONS.
    
    Given input CIFTI files for which mappings along the selected direction are
    the same type, all either series, scalars, or labels, and the other
    dimensions are index-compatible, this command concatenates the data in the
    specified indices/ranges along the selected direction (by default, on
    typical 2D cifti, concatenate horizontally, so rows become longer). The
    direction can be either an integer starting from 1, or the strings 'ROW' or
    'COLUMN'.
    
    Example: wb_command -cifti-merge out.dtseries.nii -cifti first.dtseries.nii
    -index 1 -cifti second.dtseries.nii
    
    This example would take the first column from first.dtseries.nii, followed
    by all columns from second.dtseries.nii, and write these columns to
    out.dtseries.nii. .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeOutputs`).
    """
    cifti_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_MERGE_METADATA)
    params = execution.params(params)
    cargs = cifti_merge_cargs(params, execution)
    ret = cifti_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_merge(
    cifti_out: str,
    cifti: list[CiftiMergeCiftiParamsDict] | None = None,
    limit_gb: float | None = None,
    direction: str | None = None,
    runner: Runner | None = None,
) -> CiftiMergeOutputs:
    """
    MERGE OR SPLIT ON SERIES, SCALAR, OR LABEL DIMENSIONS.
    
    Given input CIFTI files for which mappings along the selected direction are
    the same type, all either series, scalars, or labels, and the other
    dimensions are index-compatible, this command concatenates the data in the
    specified indices/ranges along the selected direction (by default, on
    typical 2D cifti, concatenate horizontally, so rows become longer). The
    direction can be either an integer starting from 1, or the strings 'ROW' or
    'COLUMN'.
    
    Example: wb_command -cifti-merge out.dtseries.nii -cifti first.dtseries.nii
    -index 1 -cifti second.dtseries.nii
    
    This example would take the first column from first.dtseries.nii, followed
    by all columns from second.dtseries.nii, and write these columns to
    out.dtseries.nii. .
    
    Args:
        cifti_out: output cifti file.
        cifti: specify an input cifti file.
        limit_gb: restrict memory used for file reading efficiency\
            \
            memory limit in gigabytes.
        direction: merge in a direction other than along rows\
            \
            the dimension to split/concatenate along, default ROW.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeOutputs`).
    """
    params = cifti_merge_params(
        cifti_out=cifti_out,
        cifti=cifti,
        limit_gb=limit_gb,
        direction=direction,
    )
    return cifti_merge_execute(params, runner)


__all__ = [
    "CIFTI_MERGE_METADATA",
    "CiftiMergeCiftiParamsDict",
    "CiftiMergeCiftiParamsDictTagged",
    "CiftiMergeIndexParamsDict",
    "CiftiMergeIndexParamsDictTagged",
    "CiftiMergeOutputs",
    "CiftiMergeParamsDict",
    "CiftiMergeParamsDictTagged",
    "CiftiMergeUpToParamsDict",
    "CiftiMergeUpToParamsDictTagged",
    "cifti_merge",
    "cifti_merge_cifti",
    "cifti_merge_execute",
    "cifti_merge_index",
    "cifti_merge_params",
    "cifti_merge_up_to",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CREATE_LABEL_METADATA = Metadata(
    id="1efacf6ed3cd2541d0611a121451c354eaf2d727.workbench",
    name="cifti-create-label",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCreateLabelVolumeParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelVolumeParamsDictNoTag', {
    "label-volume": InputPathType,
    "structure-label-volume": InputPathType,
})
CiftiCreateLabelVolumeParamsDictTagged = typing.TypedDict('CiftiCreateLabelVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "label-volume": InputPathType,
    "structure-label-volume": InputPathType,
})
CiftiCreateLabelVolumeParamsDict = _CiftiCreateLabelVolumeParamsDictNoTag | CiftiCreateLabelVolumeParamsDictTagged


_CiftiCreateLabelLeftLabelParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelLeftLabelParamsDictNoTag', {
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelLeftLabelParamsDictTagged = typing.TypedDict('CiftiCreateLabelLeftLabelParamsDictTagged', {
    "@type": typing.Literal["left-label"],
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelLeftLabelParamsDict = _CiftiCreateLabelLeftLabelParamsDictNoTag | CiftiCreateLabelLeftLabelParamsDictTagged


_CiftiCreateLabelRightLabelParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelRightLabelParamsDictNoTag', {
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelRightLabelParamsDictTagged = typing.TypedDict('CiftiCreateLabelRightLabelParamsDictTagged', {
    "@type": typing.Literal["right-label"],
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelRightLabelParamsDict = _CiftiCreateLabelRightLabelParamsDictNoTag | CiftiCreateLabelRightLabelParamsDictTagged


_CiftiCreateLabelCerebellumLabelParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelCerebellumLabelParamsDictNoTag', {
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelCerebellumLabelParamsDictTagged = typing.TypedDict('CiftiCreateLabelCerebellumLabelParamsDictTagged', {
    "@type": typing.Literal["cerebellum-label"],
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelCerebellumLabelParamsDict = _CiftiCreateLabelCerebellumLabelParamsDictNoTag | CiftiCreateLabelCerebellumLabelParamsDictTagged


_CiftiCreateLabelLabelParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelLabelParamsDictNoTag', {
    "structure": str,
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelLabelParamsDictTagged = typing.TypedDict('CiftiCreateLabelLabelParamsDictTagged', {
    "@type": typing.Literal["label"],
    "structure": str,
    "label": InputPathType,
    "roi-metric": typing.NotRequired[InputPathType | None],
})
CiftiCreateLabelLabelParamsDict = _CiftiCreateLabelLabelParamsDictNoTag | CiftiCreateLabelLabelParamsDictTagged


_CiftiCreateLabelParamsDictNoTag = typing.TypedDict('_CiftiCreateLabelParamsDictNoTag', {
    "cifti-out": str,
    "volume": typing.NotRequired[CiftiCreateLabelVolumeParamsDict | None],
    "left-label": typing.NotRequired[CiftiCreateLabelLeftLabelParamsDict | None],
    "right-label": typing.NotRequired[CiftiCreateLabelRightLabelParamsDict | None],
    "cerebellum-label": typing.NotRequired[CiftiCreateLabelCerebellumLabelParamsDict | None],
    "label": typing.NotRequired[list[CiftiCreateLabelLabelParamsDict] | None],
})
CiftiCreateLabelParamsDictTagged = typing.TypedDict('CiftiCreateLabelParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-create-label"],
    "cifti-out": str,
    "volume": typing.NotRequired[CiftiCreateLabelVolumeParamsDict | None],
    "left-label": typing.NotRequired[CiftiCreateLabelLeftLabelParamsDict | None],
    "right-label": typing.NotRequired[CiftiCreateLabelRightLabelParamsDict | None],
    "cerebellum-label": typing.NotRequired[CiftiCreateLabelCerebellumLabelParamsDict | None],
    "label": typing.NotRequired[list[CiftiCreateLabelLabelParamsDict] | None],
})
CiftiCreateLabelParamsDict = _CiftiCreateLabelParamsDictNoTag | CiftiCreateLabelParamsDictTagged


def cifti_create_label_volume(
    label_volume: InputPathType,
    structure_label_volume: InputPathType,
) -> CiftiCreateLabelVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_volume: label volume file containing the data to be copied.
        structure_label_volume: label volume file that defines which voxels to\
            use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "label-volume": label_volume,
        "structure-label-volume": structure_label_volume,
    }
    return params


def cifti_create_label_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-volume", None) is None:
        raise StyxValidationError("`label-volume` must not be None")
    if not isinstance(params["label-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-volume` has the wrong type: Received `{type(params.get("label-volume", None))}` expected `InputPathType`')
    if params.get("structure-label-volume", None) is None:
        raise StyxValidationError("`structure-label-volume` must not be None")
    if not isinstance(params["structure-label-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`structure-label-volume` has the wrong type: Received `{type(params.get("structure-label-volume", None))}` expected `InputPathType`')


def cifti_create_label_volume_cargs(
    params: CiftiCreateLabelVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        execution.input_file(params.get("label-volume", None)),
        execution.input_file(params.get("structure-label-volume", None))
    ])
    return cargs


def cifti_create_label_left_label(
    label: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateLabelLeftLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label: the label file.
        roi_metric: roi of vertices to use from left surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-label",
        "label": label,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_label_left_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelLeftLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_label_left_label_cargs(
    params: CiftiCreateLabelLeftLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-label",
        execution.input_file(params.get("label", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-left",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_label_right_label(
    label: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateLabelRightLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label: the label file.
        roi_metric: roi of vertices to use from right surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-label",
        "label": label,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_label_right_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelRightLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_label_right_label_cargs(
    params: CiftiCreateLabelRightLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-label",
        execution.input_file(params.get("label", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-right",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_label_cerebellum_label(
    label: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateLabelCerebellumLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label: the label file.
        roi_metric: roi of vertices to use from right surface\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-label",
        "label": label,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_label_cerebellum_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelCerebellumLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_label_cerebellum_label_cargs(
    params: CiftiCreateLabelCerebellumLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-label",
        execution.input_file(params.get("label", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi-cerebellum",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def cifti_create_label_label(
    structure: str,
    label: InputPathType,
    roi_metric: InputPathType | None = None,
) -> CiftiCreateLabelLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure name.
        label: the label file.
        roi_metric: roi of vertices to use from this structure\
            \
            the ROI as a metric file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "structure": structure,
        "label": label,
    }
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def cifti_create_label_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def cifti_create_label_label_cargs(
    params: CiftiCreateLabelLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        params.get("structure", None),
        execution.input_file(params.get("label", None))
    ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


class CiftiCreateLabelOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCreateLabelParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_create_label_params(
    cifti_out: str,
    volume: CiftiCreateLabelVolumeParamsDict | None = None,
    left_label: CiftiCreateLabelLeftLabelParamsDict | None = None,
    right_label: CiftiCreateLabelRightLabelParamsDict | None = None,
    cerebellum_label: CiftiCreateLabelCerebellumLabelParamsDict | None = None,
    label: list[CiftiCreateLabelLabelParamsDict] | None = None,
) -> CiftiCreateLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_label: label file for the left cortical surface.
        right_label: label file for the right cortical surface.
        cerebellum_label: label file for the cerebellum.
        label: label for a specified surface structure.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-create-label",
        "cifti-out": cifti_out,
    }
    if volume is not None:
        params["volume"] = volume
    if left_label is not None:
        params["left-label"] = left_label
    if right_label is not None:
        params["right-label"] = right_label
    if cerebellum_label is not None:
        params["cerebellum-label"] = cerebellum_label
    if label is not None:
        params["label"] = label
    return params


def cifti_create_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("volume", None) is not None:
        cifti_create_label_volume_validate(params["volume"])
    if params.get("left-label", None) is not None:
        cifti_create_label_left_label_validate(params["left-label"])
    if params.get("right-label", None) is not None:
        cifti_create_label_right_label_validate(params["right-label"])
    if params.get("cerebellum-label", None) is not None:
        cifti_create_label_cerebellum_label_validate(params["cerebellum-label"])
    if params.get("label", None) is not None:
        if not isinstance(params["label"], list):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `list[CiftiCreateLabelLabelParamsDict] | None`')
        for e in params["label"]:
            cifti_create_label_label_validate(e)


def cifti_create_label_cargs(
    params: CiftiCreateLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-create-label"
    ])
    cargs.append(params.get("cifti-out", None))
    if params.get("volume", None) is not None or params.get("left-label", None) is not None or params.get("right-label", None) is not None or params.get("cerebellum-label", None) is not None or params.get("label", None) is not None:
        cargs.extend([
            *(cifti_create_label_volume_cargs(params.get("volume", None), execution) if (params.get("volume", None) is not None) else []),
            *(cifti_create_label_left_label_cargs(params.get("left-label", None), execution) if (params.get("left-label", None) is not None) else []),
            *(cifti_create_label_right_label_cargs(params.get("right-label", None), execution) if (params.get("right-label", None) is not None) else []),
            *(cifti_create_label_cerebellum_label_cargs(params.get("cerebellum-label", None), execution) if (params.get("cerebellum-label", None) is not None) else []),
            *([a for c in [cifti_create_label_label_cargs(s, execution) for s in params.get("label", None)] for a in c] if (params.get("label", None) is not None) else [])
        ])
    return cargs


def cifti_create_label_outputs(
    params: CiftiCreateLabelParamsDict,
    execution: Execution,
) -> CiftiCreateLabelOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCreateLabelOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_create_label_execute(
    params: CiftiCreateLabelParamsDict,
    runner: Runner | None = None,
) -> CiftiCreateLabelOutputs:
    """
    CREATE A CIFTI LABEL FILE.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti. At least one component
    must be specified.
    
    The -volume option requires two volume arguments, the label-volume argument
    contains all labels you want to display (e.g. nuclei of the thalamus),
    whereas the structure-label-volume argument contains all CIFTI voxel-based
    structures you want to include data within (e.g. THALAMUS_LEFT,
    THALAMUS_RIGHT, etc). See -volume-label-import and -volume-help for format
    details of label volume files. If you just want the labels in voxels to be
    the structure names, you may use the same file for both arguments. The
    structure-label-volume must use some of the label names from this list, all
    other label names in the structure-label-volume will be ignored:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateLabelOutputs`).
    """
    cifti_create_label_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_LABEL_METADATA)
    params = execution.params(params)
    cargs = cifti_create_label_cargs(params, execution)
    ret = cifti_create_label_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_create_label(
    cifti_out: str,
    volume: CiftiCreateLabelVolumeParamsDict | None = None,
    left_label: CiftiCreateLabelLeftLabelParamsDict | None = None,
    right_label: CiftiCreateLabelRightLabelParamsDict | None = None,
    cerebellum_label: CiftiCreateLabelCerebellumLabelParamsDict | None = None,
    label: list[CiftiCreateLabelLabelParamsDict] | None = None,
    runner: Runner | None = None,
) -> CiftiCreateLabelOutputs:
    """
    CREATE A CIFTI LABEL FILE.
    
    All input files must have the same number of columns/subvolumes. Only the
    specified components will be in the output cifti. At least one component
    must be specified.
    
    The -volume option requires two volume arguments, the label-volume argument
    contains all labels you want to display (e.g. nuclei of the thalamus),
    whereas the structure-label-volume argument contains all CIFTI voxel-based
    structures you want to include data within (e.g. THALAMUS_LEFT,
    THALAMUS_RIGHT, etc). See -volume-label-import and -volume-help for format
    details of label volume files. If you just want the labels in voxels to be
    the structure names, you may use the same file for both arguments. The
    structure-label-volume must use some of the label names from this list, all
    other label names in the structure-label-volume will be ignored:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti_out: the output cifti file.
        volume: volume component.
        left_label: label file for the left cortical surface.
        right_label: label file for the right cortical surface.
        cerebellum_label: label file for the cerebellum.
        label: label for a specified surface structure.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateLabelOutputs`).
    """
    params = cifti_create_label_params(
        cifti_out=cifti_out,
        volume=volume,
        left_label=left_label,
        right_label=right_label,
        cerebellum_label=cerebellum_label,
        label=label,
    )
    return cifti_create_label_execute(params, runner)


__all__ = [
    "CIFTI_CREATE_LABEL_METADATA",
    "CiftiCreateLabelCerebellumLabelParamsDict",
    "CiftiCreateLabelCerebellumLabelParamsDictTagged",
    "CiftiCreateLabelLabelParamsDict",
    "CiftiCreateLabelLabelParamsDictTagged",
    "CiftiCreateLabelLeftLabelParamsDict",
    "CiftiCreateLabelLeftLabelParamsDictTagged",
    "CiftiCreateLabelOutputs",
    "CiftiCreateLabelParamsDict",
    "CiftiCreateLabelParamsDictTagged",
    "CiftiCreateLabelRightLabelParamsDict",
    "CiftiCreateLabelRightLabelParamsDictTagged",
    "CiftiCreateLabelVolumeParamsDict",
    "CiftiCreateLabelVolumeParamsDictTagged",
    "cifti_create_label",
    "cifti_create_label_cerebellum_label",
    "cifti_create_label_execute",
    "cifti_create_label_label",
    "cifti_create_label_left_label",
    "cifti_create_label_params",
    "cifti_create_label_right_label",
    "cifti_create_label_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_LABEL_ADJACENCY_METADATA = Metadata(
    id="2e19cd8506381d00a3254296038c79a01dddd353.workbench",
    name="cifti-label-adjacency",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiLabelAdjacencyParamsDictNoTag = typing.TypedDict('_CiftiLabelAdjacencyParamsDictNoTag', {
    "label-in": InputPathType,
    "adjacency-out": str,
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
})
CiftiLabelAdjacencyParamsDictTagged = typing.TypedDict('CiftiLabelAdjacencyParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-label-adjacency"],
    "label-in": InputPathType,
    "adjacency-out": str,
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
    "surface": typing.NotRequired[InputPathType | None],
})
CiftiLabelAdjacencyParamsDict = _CiftiLabelAdjacencyParamsDictNoTag | CiftiLabelAdjacencyParamsDictTagged


class CiftiLabelAdjacencyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiLabelAdjacencyParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    adjacency_out: OutputPathType
    """the output cifti pconn adjacency matrix"""


def cifti_label_adjacency_params(
    label_in: InputPathType,
    adjacency_out: str,
    surface: InputPathType | None = None,
    surface_: InputPathType | None = None,
    surface_2: InputPathType | None = None,
) -> CiftiLabelAdjacencyParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: the input cifti label file.
        adjacency_out: the output cifti pconn adjacency matrix.
        surface: specify the cerebellum surface to use\
            \
            the cerebellum surface file.
        surface_: specify the right surface to use\
            \
            the right surface file.
        surface_2: specify the left surface to use\
            \
            the left surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-label-adjacency",
        "label-in": label_in,
        "adjacency-out": adjacency_out,
    }
    if surface is not None:
        params["surface"] = surface
    if surface_ is not None:
        params["surface"] = surface_
    if surface_2 is not None:
        params["surface"] = surface_2
    return params


def cifti_label_adjacency_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelAdjacencyParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("adjacency-out", None) is None:
        raise StyxValidationError("`adjacency-out` must not be None")
    if not isinstance(params["adjacency-out"], str):
        raise StyxValidationError(f'`adjacency-out` has the wrong type: Received `{type(params.get("adjacency-out", None))}` expected `str`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')
    if params.get("surface", None) is not None:
        if not isinstance(params["surface"], (pathlib.Path, str)):
            raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType | None`')


def cifti_label_adjacency_cargs(
    params: CiftiLabelAdjacencyParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-label-adjacency"
    ])
    cargs.append(execution.input_file(params.get("label-in", None)))
    cargs.append(params.get("adjacency-out", None))
    if params.get("surface", None) is not None:
        cargs.extend([
            "-cerebellum-surface",
            execution.input_file(params.get("surface", None))
        ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-right-surface",
            execution.input_file(params.get("surface", None))
        ])
    if params.get("surface", None) is not None:
        cargs.extend([
            "-left-surface",
            execution.input_file(params.get("surface", None))
        ])
    return cargs


def cifti_label_adjacency_outputs(
    params: CiftiLabelAdjacencyParamsDict,
    execution: Execution,
) -> CiftiLabelAdjacencyOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelAdjacencyOutputs(
        root=execution.output_file("."),
        adjacency_out=execution.output_file(params.get("adjacency-out", None)),
    )
    return ret


def cifti_label_adjacency_execute(
    params: CiftiLabelAdjacencyParamsDict,
    runner: Runner | None = None,
) -> CiftiLabelAdjacencyOutputs:
    """
    MAKE ADJACENCY MATRIX OF A CIFTI LABEL FILE.
    
    Find face-adjacent voxels and connected vertices that have different label
    values, and count them for each pair. Put the resulting counts into a
    parcellated connectivity file, with the diagonal being zero. This gives a
    rough estimate of how long or expansive the border between two labels is.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelAdjacencyOutputs`).
    """
    cifti_label_adjacency_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_LABEL_ADJACENCY_METADATA)
    params = execution.params(params)
    cargs = cifti_label_adjacency_cargs(params, execution)
    ret = cifti_label_adjacency_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_label_adjacency(
    label_in: InputPathType,
    adjacency_out: str,
    surface: InputPathType | None = None,
    surface_: InputPathType | None = None,
    surface_2: InputPathType | None = None,
    runner: Runner | None = None,
) -> CiftiLabelAdjacencyOutputs:
    """
    MAKE ADJACENCY MATRIX OF A CIFTI LABEL FILE.
    
    Find face-adjacent voxels and connected vertices that have different label
    values, and count them for each pair. Put the resulting counts into a
    parcellated connectivity file, with the diagonal being zero. This gives a
    rough estimate of how long or expansive the border between two labels is.
    
    Args:
        label_in: the input cifti label file.
        adjacency_out: the output cifti pconn adjacency matrix.
        surface: specify the cerebellum surface to use\
            \
            the cerebellum surface file.
        surface_: specify the right surface to use\
            \
            the right surface file.
        surface_2: specify the left surface to use\
            \
            the left surface file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelAdjacencyOutputs`).
    """
    params = cifti_label_adjacency_params(
        label_in=label_in,
        adjacency_out=adjacency_out,
        surface=surface,
        surface_=surface_,
        surface_2=surface_2,
    )
    return cifti_label_adjacency_execute(params, runner)


__all__ = [
    "CIFTI_LABEL_ADJACENCY_METADATA",
    "CiftiLabelAdjacencyOutputs",
    "CiftiLabelAdjacencyParamsDict",
    "CiftiLabelAdjacencyParamsDictTagged",
    "cifti_label_adjacency",
    "cifti_label_adjacency_execute",
    "cifti_label_adjacency_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_VECTOR_OPERATION_METADATA = Metadata(
    id="2ed0edf6d0f6355b8d747c848ce1f1d204659400.workbench",
    name="metric-vector-operation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricVectorOperationParamsDictNoTag = typing.TypedDict('_MetricVectorOperationParamsDictNoTag', {
    "vectors-a": InputPathType,
    "vectors-b": InputPathType,
    "operation": str,
    "metric-out": str,
    "magnitude": bool,
    "normalize-output": bool,
    "normalize-b": bool,
    "normalize-a": bool,
})
MetricVectorOperationParamsDictTagged = typing.TypedDict('MetricVectorOperationParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-vector-operation"],
    "vectors-a": InputPathType,
    "vectors-b": InputPathType,
    "operation": str,
    "metric-out": str,
    "magnitude": bool,
    "normalize-output": bool,
    "normalize-b": bool,
    "normalize-a": bool,
})
MetricVectorOperationParamsDict = _MetricVectorOperationParamsDictNoTag | MetricVectorOperationParamsDictTagged


class MetricVectorOperationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricVectorOperationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output file"""


def metric_vector_operation_params(
    vectors_a: InputPathType,
    vectors_b: InputPathType,
    operation: str,
    metric_out: str,
    magnitude: bool = False,
    normalize_output: bool = False,
    normalize_b: bool = False,
    normalize_a: bool = False,
) -> MetricVectorOperationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        vectors_a: first vector input file.
        vectors_b: second vector input file.
        operation: what vector operation to do.
        metric_out: the output file.
        magnitude: output the magnitude of the result (not valid for dot\
            product).
        normalize_output: normalize output vectors (not valid for dot product).
        normalize_b: normalize vectors of second input.
        normalize_a: normalize vectors of first input.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-vector-operation",
        "vectors-a": vectors_a,
        "vectors-b": vectors_b,
        "operation": operation,
        "metric-out": metric_out,
        "magnitude": magnitude,
        "normalize-output": normalize_output,
        "normalize-b": normalize_b,
        "normalize-a": normalize_a,
    }
    return params


def metric_vector_operation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricVectorOperationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("vectors-a", None) is None:
        raise StyxValidationError("`vectors-a` must not be None")
    if not isinstance(params["vectors-a"], (pathlib.Path, str)):
        raise StyxValidationError(f'`vectors-a` has the wrong type: Received `{type(params.get("vectors-a", None))}` expected `InputPathType`')
    if params.get("vectors-b", None) is None:
        raise StyxValidationError("`vectors-b` must not be None")
    if not isinstance(params["vectors-b"], (pathlib.Path, str)):
        raise StyxValidationError(f'`vectors-b` has the wrong type: Received `{type(params.get("vectors-b", None))}` expected `InputPathType`')
    if params.get("operation", None) is None:
        raise StyxValidationError("`operation` must not be None")
    if not isinstance(params["operation"], str):
        raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("magnitude", False) is None:
        raise StyxValidationError("`magnitude` must not be None")
    if not isinstance(params["magnitude"], bool):
        raise StyxValidationError(f'`magnitude` has the wrong type: Received `{type(params.get("magnitude", False))}` expected `bool`')
    if params.get("normalize-output", False) is None:
        raise StyxValidationError("`normalize-output` must not be None")
    if not isinstance(params["normalize-output"], bool):
        raise StyxValidationError(f'`normalize-output` has the wrong type: Received `{type(params.get("normalize-output", False))}` expected `bool`')
    if params.get("normalize-b", False) is None:
        raise StyxValidationError("`normalize-b` must not be None")
    if not isinstance(params["normalize-b"], bool):
        raise StyxValidationError(f'`normalize-b` has the wrong type: Received `{type(params.get("normalize-b", False))}` expected `bool`')
    if params.get("normalize-a", False) is None:
        raise StyxValidationError("`normalize-a` must not be None")
    if not isinstance(params["normalize-a"], bool):
        raise StyxValidationError(f'`normalize-a` has the wrong type: Received `{type(params.get("normalize-a", False))}` expected `bool`')


def metric_vector_operation_cargs(
    params: MetricVectorOperationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-vector-operation"
    ])
    cargs.append(execution.input_file(params.get("vectors-a", None)))
    cargs.append(execution.input_file(params.get("vectors-b", None)))
    cargs.append(params.get("operation", None))
    cargs.append(params.get("metric-out", None))
    if params.get("magnitude", False):
        cargs.append("-magnitude")
    if params.get("normalize-output", False):
        cargs.append("-normalize-output")
    if params.get("normalize-b", False):
        cargs.append("-normalize-b")
    if params.get("normalize-a", False):
        cargs.append("-normalize-a")
    return cargs


def metric_vector_operation_outputs(
    params: MetricVectorOperationParamsDict,
    execution: Execution,
) -> MetricVectorOperationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricVectorOperationOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_vector_operation_execute(
    params: MetricVectorOperationParamsDict,
    runner: Runner | None = None,
) -> MetricVectorOperationOutputs:
    """
    DO A VECTOR OPERATION ON METRIC FILES.
    
    Does a vector operation on two metric files (that must have a multiple of 3
    columns). Either of the inputs may have multiple vectors (more than 3
    columns), but not both (at least one must have exactly 3 columns). The
    -magnitude and -normalize-output options may not be specified together, or
    with an operation that returns a scalar (dot product). The <operation>
    parameter must be one of the following:
    
    DOT
    CROSS
    ADD
    SUBTRACT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricVectorOperationOutputs`).
    """
    metric_vector_operation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_VECTOR_OPERATION_METADATA)
    params = execution.params(params)
    cargs = metric_vector_operation_cargs(params, execution)
    ret = metric_vector_operation_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_vector_operation(
    vectors_a: InputPathType,
    vectors_b: InputPathType,
    operation: str,
    metric_out: str,
    magnitude: bool = False,
    normalize_output: bool = False,
    normalize_b: bool = False,
    normalize_a: bool = False,
    runner: Runner | None = None,
) -> MetricVectorOperationOutputs:
    """
    DO A VECTOR OPERATION ON METRIC FILES.
    
    Does a vector operation on two metric files (that must have a multiple of 3
    columns). Either of the inputs may have multiple vectors (more than 3
    columns), but not both (at least one must have exactly 3 columns). The
    -magnitude and -normalize-output options may not be specified together, or
    with an operation that returns a scalar (dot product). The <operation>
    parameter must be one of the following:
    
    DOT
    CROSS
    ADD
    SUBTRACT.
    
    Args:
        vectors_a: first vector input file.
        vectors_b: second vector input file.
        operation: what vector operation to do.
        metric_out: the output file.
        magnitude: output the magnitude of the result (not valid for dot\
            product).
        normalize_output: normalize output vectors (not valid for dot product).
        normalize_b: normalize vectors of second input.
        normalize_a: normalize vectors of first input.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricVectorOperationOutputs`).
    """
    params = metric_vector_operation_params(
        vectors_a=vectors_a,
        vectors_b=vectors_b,
        operation=operation,
        metric_out=metric_out,
        magnitude=magnitude,
        normalize_output=normalize_output,
        normalize_b=normalize_b,
        normalize_a=normalize_a,
    )
    return metric_vector_operation_execute(params, runner)


__all__ = [
    "METRIC_VECTOR_OPERATION_METADATA",
    "MetricVectorOperationOutputs",
    "MetricVectorOperationParamsDict",
    "MetricVectorOperationParamsDictTagged",
    "metric_vector_operation",
    "metric_vector_operation_execute",
    "metric_vector_operation_params",
]
